#!/usr/bin/env python3
# script to produce torque balance curves for Scorpius X-1

# Copyright (C) 2022 Brynmor Haskell, John T. Whelan

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with with program; see the file COPYING. If not, see
# <https://www.gnu.org/licenses/>.
import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate

## constants

# G=6.67e-8
G=6.6743e-8 # Newton's constant in cgs
# c=2.99e10
mSec = 1. / 299792458. # one meter, in seconds
c = 100 / mSec # speed of light in cm/sec (cgs)
yrSec = 365.25 * 24 * 60 * 60 # One Julian year in seconds
AUcgs = 100 * 149_597_870_700 # one astronomical unit in cm (cgs)
kpccgs = 10**3 * 360 * 60 * 60 / (2*np.pi) * AUcgs # one kiloparsec in cm (cgs)

## distance
# dd=2.8*3.0857e21
dd=2.8*kpccgs

### Xrayflux
Fx=3.9e-7

## efficiency
X=1e0

## radius

R=1e6

##msun

msun=1.989e33
## mass

M=1.4e0*msun

##mdot

mdot=4e0*np.pi*dd**2*Fx*R/X/G/M

## B field (units 10e8)

B=10e0

## simple torque balance assuming Ra=R, as in previous LVC searches

def tbnaive(f,xiv=1):
    xi=xiv
    ##ra
    ra=35e5*xi*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*B**(4e0/7e0)
#    print('rA=%g'%ra)
    if ra<R: ra=R
    ##rc
    ## f is GW frequency, so has factor of 1/2 as the expression in GS2021 uses f_spin
    rc=27e5*(f/500e0/2e0)**(-2e0/3e0)

    ## fastness
    omega=(ra/rc)**(3e0/2e0)
    
    #h0
    h02=10e0*Fx/X*R*R**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)
    h0=h02**(0.5e0)
#    print(h0)
    
    ## plot GW freq vs. torque balance h0

    return h0

### Alfven radius model as in Zhang et al. AEI 02 search (definitions as in Glampedakis and Suvorov 2021)
def tbra(f,xiv):
    xi=xiv
    ##ra
    ra=35e5*xi*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*B**(4e0/7e0)
    if ra<R: ra=R
    ##rc
    rc=27e5*(f/500e0/2e0)**(-2e0/3e0)

    ## fastness
    omega=(ra/rc)**(3e0/2e0)
    
    #h0
    h02=10e0*Fx/X*R*ra**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)
    h0=h02**(0.5e0)
#    print(h0)
    
    return h0

### Glampedakis and Suvorov 2021 model New2    
def tbt2(f,xiv):
    xi=xiv
    ##ra
    ra=35e5*xi*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*(B)**(4e0/7e0)
    if ra<R: ra=R
    ##rc
    rc=27e5*(f/500e0/2e0)**(-2e0/3e0)

    ## fastness
    omega=(ra/rc)**(3e0/2e0)
    
    #h0
    h02=10e0*Fx/X*R*ra**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)*2e0/3e0*xi**(-7e0/2e0)*((1e0+3e0*xi**(7e0/2e0))/2e0-omega+omega**2/3e0)
    h0=h02**(0.5e0)
    return h0

### Glampedakis and Suvorov 2021 model New3
def tbt3(f,xiv):
    xi=xiv
    ##ra
    ra=35e5*xi*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*B**(4e0/7e0)
    if ra<R: ra=R
    ##rc
    rc=27e5*(f/500e0/2e0)**(-2e0/3e0)

    ## fastness
    omega=(ra/rc)**(3e0/2e0)
    
    #h0
    h02=10e0*Fx/X*R*ra**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)*1e0/3e0*xi**(-7e0/2e0)*((1e0+3e0*xi**(7e0/2e0))-2*omega)
    h0=h02**(0.5e0)
#    print(h0)
    return h0
    
#### range for B=10e9  assuming B fixed to B=10e9
def tbrange(f):
    xi1=0.3
    xi2=0.5
    ##ra for 0.1
    ra1=35e5*xi1*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*B**(4e0/7e0)
    if ra1<R: ra1=R
    ##ra for 0.5
    ra2=35e5*xi2*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*B**(4e0/7e0)
    if ra2<R: ra2=R
        
    ##rc
    rc=27e5*(f/500e0/2e0)**(-2e0/3e0)

    ## fastness
    omega1=(ra1/rc)**(3e0/2e0)
    omega2=(ra2/rc)**(3e0/2e0)
    
    #h0 for 0.1
    h021=10e0*Fx/X*R*ra1**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)*1e0/3e0*xi1**(-7e0/2e0)*((1e0+3e0*xi1**(7e0/2e0))-2*omega1)
    ## avoid negative numbers in the root. Set hplot=hminplot**2 where hminplot is lower than the minimum value plotted on the y axis
    
    h01=h021**(0.5e0)

    #h0 for 0.5
    h022=10e0*Fx/X*R*ra2**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)*1e0/3e0*xi2**(-7e0/2e0)*((1e0+3e0*xi2**(7e0/2e0))-2*omega2)
#    print(h022)

    h022pos=np.maximum(h022,1e-53)
    h02=h022pos**(0.5e0)
    ### naive
    h0n=3.4e-26*(600/f)**(0.5e0)

    #### Alfven radius with 0.1
    xi=0.1
    ##ra
    ra=35e5*xi*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(R/1e6)**(12e0/7e0)*B**(4e0/7e0)
    if ra<R: ra=R
    ##rc
    rc=27e5*(f/500e0/2e0)**(-2e0/3e0)

    ## fastness
    omega=(ra/rc)**(3e0/2e0)
    
    #h0
    h02a=10e0*Fx/X*R*ra**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)
    h0a=h02a**(0.5e0)
    
    ## calculate hmin
    #     hmin=np.minimum(h02,h0a)
    hmin=h02
    return (hmin,h01)

############################
## 10^8 G field, assuming B fixed to B=10e9
def tbrange8(f):
    xi1=0.3
    xi2=0.5
    ##ra for 0.1
    ra1=35e5*xi1*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*(B/10e0)**(4e0/7e0)
    if ra1<R: ra1=R
    ##ra for 0.5
    ra2=35e5*xi2*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*(B/10e0)**(4e0/7e0)
    if ra2<R: ra2=R
        
    ##rc
    rc=27e5*(f/500e0/2e0)**(-2e0/3e0)

    ## fastness
    omega1=(ra1/rc)**(3e0/2e0)
    omega2=(ra2/rc)**(3e0/2e0)
    
    #h0 for 0.1
    h021=10e0*Fx/X*R*ra1**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)*1e0/3e0*xi1**(-7e0/2e0)*((1e0+3e0*xi1**(7e0/2e0))-2*omega1)
    ## avoid negative numbers in the root. Set hplot=hminplot**2 where hminplot is lower than the minimum value plotted on the y axis
    
    h01=h021**(0.5e0)

    #h0 for 0.5
    h022=10e0*Fx/X*R*ra2**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)*1e0/3e0*xi2**(-7e0/2e0)*((1e0+3e0*xi2**(7e0/2e0))-2*omega2)
#    print(h022)

    h022pos=np.maximum(h022,1e-53)
    h02=h022pos**(0.5e0)
    ### naive
    h0n=3.4e-26*(600/f)**(0.5e0)

    #### Alfven radius with 0.1
    xi=0.1
    ##ra
    ra=35e5*xi*(mdot/(10e-10*msun/yrSec))**(-2e0/7e0)*(M/1.4e0/msun)**(-1e0/7e0)*(R/1e6)**(12e0/7e0)*B**(4e0/7e0)
    if ra<R: ra=R
    ##rc
    rc=27e5*(f/500e0/2e0)**(-2e0/3e0)

    ## fastness
    omega=(ra/rc)**(3e0/2e0)
    
    #h0
    h02a=10e0*Fx/X*R*ra**(0.5e0)/c**3/f*G**(0.5e0)/M**(0.5e0)
    h0a=h02a**(0.5e0)

    ## calculate hmin
    #     hmin=np.minimum(h02,h0a)
    hmin=h02

    ## plot GW freq vs. torque balance h0
    return (hmin,h01)

     ############################################




# #plt.text(200,2e-26,'B=$10^9$',fontsize=18)


# #plot_ra(1,2,'dotted','orange',r'R_alfven')
# #plot_ra(0.1,2,'dotted','blue',r'R_alfven')
# plot_naive(1,3, 'dashed','blue',r'Surface accretion')
# #plot_t2(0.1,2, 'dotted','magenta',r'$\xi=0.1$')
# #plot_t2(0.5,2, 'dotted','black',r'$\xi=0.5$')
# #plot_t3(0.1,2, 'dotted','green',r'GS_3, $\xi=0.1$')

# plot_range(1, 'dashed','black',r'B=$10^9$ G')
# plot_range8(1, 'dashed','darkred',r'B=$10^8$ G')

# plt.legend(loc='upper right', ncol=2,fontsize='small')

def masslimit(f,h,eosfilename):

    ### read in mass and radius from the Eos. Radius is in km and mass in solar masses.
    r1, m1 = np.loadtxt(eosfilename, comments='#',  usecols=(0,1), unpack=True)

    ## max mass

    M1max= np.asarray(m1).max(0)

    rr = interpolate.interp1d(m1, r1)

    ### calculate (R_10)^(3/4)/M_1.4^(1/4)

    C1=(rr(m1)/10e0)**(3e0/4e0)/(m1/1.4e0)**(1e0/4e0)

    C1max= np.asarray(C1).max(0)
    C1min= np.asarray(C1).min(0)

    g = interpolate.interp1d(C1, m1)

    Clim = h/(3.4e-26)*(f/600e0)**(0.5e0)

    #restrict to limits provided by eos

    for i in range(0, len(Clim)):    
        if Clim[i]<C1min: Clim[i]=C1min
        if Clim[i]>C1max: Clim[i]=C1max

    # make this a limit on m

    mlim= g(Clim)

    return((mlim,M1max))

def cosilimit(f,h,eosfilename,M=1.4):

    ### read in mass and radius from the Eos. Radius is in km and mass in solar masses.
    r1, m1 = np.loadtxt(eosfilename, comments='#',  usecols=(0,1), unpack=True)

    ## max mass

    M1max= np.asarray(m1).max(0)

    rr = interpolate.interp1d(m1, r1)


    ## radius from EOS

    rr14 = rr(M)

    
    ### h0 naive for mass

    h0=5.48e-27*(Fx/1e-8)**(0.5e0)*(rr14/10e0)**(0.75e0)/(f/600e0)**(0.5e0)*(M/1.4e0)**(0.25e0)



    ### ratio 2h_eff/h_0

    alpha=2e0*h/h0-0.25e0



    ## caluclate limit, first term

    t1=np.sqrt(4e0*alpha+9e0)
    ## total

    limit=np.sqrt(t1-3e0)

    return(limit)
